"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadMongoDb = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const tar_1 = __importDefault(require("tar"));
const util_1 = require("util");
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const download_1 = __importDefault(require("download"));
const stream_1 = require("stream");
const mongodb_download_url_1 = __importDefault(require("mongodb-download-url"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('mongodb-downloader');
async function downloadMongoDb(tmpdir, targetVersionSemverSpecifier = '*', options = {}) {
    var _a;
    let wantsEnterprise = (_a = options.enterprise) !== null && _a !== void 0 ? _a : false;
    async function lookupDownloadUrl() {
        return (await (0, mongodb_download_url_1.default)({
            version: targetVersionSemverSpecifier,
            enterprise: wantsEnterprise,
            ...options,
        })).url;
    }
    await fs_1.promises.mkdir(tmpdir, { recursive: true });
    if (targetVersionSemverSpecifier === 'latest-alpha') {
        return await doDownload(tmpdir, !!options.crypt_shared, 'latest-alpha', lookupDownloadUrl);
    }
    if (/-enterprise$/.test(targetVersionSemverSpecifier)) {
        wantsEnterprise = true;
        targetVersionSemverSpecifier = targetVersionSemverSpecifier.replace(/-enterprise$/, '');
    }
    return await doDownload(tmpdir, !!options.crypt_shared, targetVersionSemverSpecifier +
        (wantsEnterprise ? '-enterprise' : '-community'), () => lookupDownloadUrl());
}
exports.downloadMongoDb = downloadMongoDb;
const downloadPromises = {};
async function doDownload(tmpdir, isCryptLibrary, version, lookupDownloadUrl) {
    var _a;
    const downloadTarget = path_1.default.resolve(tmpdir, `mongodb-${process.platform}-${process.env.DISTRO_ID || 'none'}-${process.arch}-${version}`.replace(/[^a-zA-Z0-9_-]/g, ''));
    return ((_a = downloadPromises[downloadTarget]) !== null && _a !== void 0 ? _a : (downloadPromises[downloadTarget] = (async () => {
        const bindir = path_1.default.resolve(downloadTarget, isCryptLibrary && process.platform !== 'win32' ? 'lib' : 'bin');
        try {
            await fs_1.promises.stat(bindir);
            debug(`Skipping download because ${downloadTarget} exists`);
            return bindir;
        }
        catch (_a) {
        }
        await fs_1.promises.mkdir(downloadTarget, { recursive: true });
        const url = await lookupDownloadUrl();
        debug(`Downloading ${url} into ${downloadTarget}...`);
        async function downloadAndExtract(withExtraStripDepth = 0) {
            if (/\.tgz$|\.tar(\.[^.]+)?$/.exec(url)) {
                const response = await (0, node_fetch_1.default)(url);
                await (0, util_1.promisify)(stream_1.pipeline)(response.body, tar_1.default.x({ cwd: downloadTarget, strip: isCryptLibrary ? 0 : 1 }));
            }
            else {
                await (0, download_1.default)(url, downloadTarget, {
                    extract: true,
                    strip: isCryptLibrary ? 0 : 1,
                });
            }
            try {
                await fs_1.promises.stat(bindir);
            }
            catch (err) {
                if (withExtraStripDepth === 0 && url.includes('macos')) {
                    console.info('Retry due to miscalculated --strip-components depth');
                    return await downloadAndExtract(1);
                }
                throw err;
            }
        }
        await downloadAndExtract();
        debug(`Download complete`, bindir);
        return bindir;
    })()));
}
//# sourceMappingURL=index.js.map